_name_map = {
    "age": "Age",
    "albumin": "Albumin",
    "alt": "ALT",
    "ast": "AST",
    "bonet": "T-score",
    "bonez": "Z-score",
    "bparmc": "팔둘레",
    "bmi": "BMI",
    "bun": "BUN",
    "creatine": "Creatine",
    "crp": "CRP",
    "fef25": "FEF25",
    "fev1": "FEV1",
    "fvc": "FVC",
    # "gripl1": "",
    "gripr1": "악력",
    # "grwhich": "",
    "hb": "Hb",
    "hct": "Hct",
    "hdl": "HDL",
    "hba1c": "HbA1C",
    "glu0": "혈당",
    "height": "키",
    "hip": "엉덩이 둘레",
    "waist": "허리둘레",
    "labi": "ABI(왼쪽)",
    "rabi": "ABI(오른쪽)",
    "ldl": "LDL",
    "mch": "MCH",
    "mchc": "MCHC",
    "mcv": "MCV",
    "muscle": "근육량",
    "plat": "PLT",
    "pulse": "맥박",
    "r_gtp": "γ-GTP",
    "rbc": "RBC",
    "stiffness": "Stiffness",
    "t_bil": "T.Bilirubin",
    "tchl": "Cholesterol",
    "tg": "TG",
    "u_ket": "U.ketone",
    "u_ph": "U.pH",
    "uricacid": "UA",
    "wbc": "WBC",
    "whr": "WHR",
    "weight": "체중",
    "sinus bradycardia": "sinus brady",
    "ekg": "EKG",
    "code1": "EKG소견",
    "dm": "당뇨",
}

_ekg_map = {
    "RAE": "RAH",
    "LAE": "LAH",
    "LVH by voltage criteria": "LVH",
    "LVH by cornell voltage": "LVH",
    "LVH with strain": "LVH",
    # A.fib.
    "A.fib.": "A fib",
    "A.fib.": "A fib",
    "A fib with controlled VR": "A fib",
    "A fib with rapid VR": "A fib",
    "A fib with slow VR": "A fib",
    "A.fib. With controlled VR": "A fib",
    "A.fib. With rapid VR": "A fib",
    "A.fib. With slow VR": "A fib",
    "A.fib.with marked slow VR": "A fib",
    # premature
    "APC": "premature",
    "APC bigeminy": "premature",
    "APC or JPC": "premature",
    "VPC": "premature",
    "VPC bigeminy": "premature",
    "VPC couplet": "premature",
    "VPC trigeminy": "premature",
    "VPCs quadrigeminy": "premature",
    "VPCs quardisgeminy": "premature",
    # pre-excitation
    "LGL syndrome": "pre-excitation",
    "preexcitation": "pre-excitation",
    "preexitation": "pre-excitation",
    "WPW syndrome": "pre-excitation",
    "WPW sydrome": "pre-excitation",
    "ventricular preexcitation": "pre-excitation",
    "ventricular preexitation": "pre-excitation",
    "ventriculat preexitation": "pre-excitation",
    # MI
    "old MI anteroseptal": "MI",
    "old MI inferior": "MI",
    "old MI lateral": "MI",
    "old MI posterior": "MI",
    "old MI septal": "MI",
    "old MI, anterior": "MI",
    "old MIf, anterior(QS pattern in all of leads V1, V2, and V3)": "MI",
    "old MIf, anterior(QS pattern in all of leads V1-V4 or V1-V5)": "MI",
    "old Mif anteroseptal": "MI",
    "old Mif inferior": "MI",
    "old Mif lateral": "MI",
    "old Mif posterior": "MI",
    "old Mif septal": "MI",
    "myocardial injury anterior wall": "MI",
    "myocardial ischemia": "MI",
    "myocardial ischemia(MIs)(anterior  wall)": "MI",
    "myocardial ischemia(MIs)(anteroseptal wall)": "MI",
    "myocardial ischemia(MIs)(inferior wall)": "MI",
    "MIs(mycardial ischemia)": "MI",
    "MIs(myocardial ischemia)(lateral ischemia)": "MI",
    "Mif(mycardial infarction)": "MI",
    "r/o) old MI(posterior wall)": "MI",
    "r/o) old MIf(myocardial infarction)(posterior wall)": "MI",
    "extensive anterior MI": "MI",
    "extensive anterior MIf": "MI",
    # sinus arrhythmia
    "sinus bradycardia": "sinus arrhythmia",
    "sinus bradycardia with junctional escape beat": "sinus arrhythmia",
    "sinus tachycardia": "sinus arrhythmia",
    "marked sinus bradycardia": "sinus arrhythmia",
    "marked sinus bradycardia with junctional escape beat": "sinus arrhythmia",
    # BBB
    "LBBB": "BBB",
    "RBBB": "BBB",
    "incomplete LBBB": "BBB",
    "incomplete RBBB": "BBB",
    "complete LBBB": "BBB",
    "complete RBBB": "BBB",
    # Heart block
    "AV dissociation": "block",
    "High grade AV block": "block",
    "complete AV block": "block",
    "first degree AV block": "block",
    "secondary degree AV block (type 1 -partial)": "block",
    "Advanced AV block(2:1 conduction)": "block",
    "Complete  heart block with junctinal rhythem": "block",
    "Complete  heart block with junctional rhythem": "block",
    "bifascicular block": "block",
    "left anterior hemiblock": "block",
    "sinoatrial block": "block",
    "sionatrial block": "block",
    "trigascifulcar black(三支阻滞）": "block",
    "LPFB": "block",
    # arrest
    "sinus arrest with escape beats": "arrest",
    "sinus arrest with junctional escape": "arrest",
    "sinus arrest with nunctional escape": "arrest",
    "sinus arrest with wandering atrial pacemaker": "arrest",
    # AFL
    "atrial Filtter with 4:1 conduction": "AFL",
    "atrial filtter with variable conduction": "AFL",
    "atrial flutter with 4:1 conduction": "AFL",
    "atrial flutter with variable conduction": "AFL",
    # ST-T change
    "brugada pattern": "ST-T change",
    "nonspecific ST-T change": "ST-T change",
    "r/o)brugada pattern": "ST-T change",
    # 오타
}



@staticmethod
def __readCodingBook(filePath):
    from pykoges.datatype import Question
    import csv, openpyxl

    questions_list = []
    # 파일읽기
    wb = openpyxl.Workbook()
    db = wb.active
    with open(filePath, encoding="utf-8") as f:
        reader = csv.reader(f, delimiter=",", quotechar='"')
        for row in reader:
            db.append(row)

    question = None
    # 열의 개수가 1이상이고 두번째 행, 첫번째 열의 데이터가 존재하는 경우
    if db.max_column > 0 and db.max_row > 0:
        for row in db.iter_rows(2):
            # openpyxl 라이브러리는 cell.value를 통해 값을 호출하므로 값의 리스트를 가져오도록 설정
            row = [x.value for x in row]
            # 첫행인 경우 통과
            if str.startswith(row[0], "설문지명"):
                continue
            # 행의 8개 데이터중 어떤 것이라도 있는 경우 (질문 정보)
            elif any(row[:8]):
                # 행 데이터를 바탕으로 질문 생성
                question = Question.from_row(row)
                # 파일 정보 추가
                question.add_fileinfo(filePath)
                # 전체 질문 목록에 추가
                questions_list.append(question)
            # 행의 8개 데이터가 모두 빈 경우 경우 (질문 선지)
            elif question:
                # 설정된 질문에 답변 추가
                question.add_answer(row)
    return questions_list


def read(folderName="./data_fixed"):
    from pykoges.datatype import Questions
    from tqdm.notebook import tqdm
    import os, inspect

    basepath = os.path.dirname(inspect.stack()[1].filename)
    folder = os.path.join(basepath, folderName)
    if not os.path.exists(folder):
        raise FileExistsError("파일을 읽어올 경로가 존재하지 않습니다.\n폴더 이름을 다시 설정해주세요.")
    # 중복실행을 대비해 초기 변수들을 비워줍니다.
    questions_list = []
    # 확장자가 없거나 (폴더)
    # 엑셀을 실행시켰을 때 생기는 임시파일 (~$...)인경우 통과
    files = filter(
        lambda x: os.path.splitext(x)[1]
        and not x.startswith("~")
        and "codingbook" in os.path.splitext(x)[0],
        os.listdir(folder),
    )
    for x in tqdm(list(files), desc="코딩북 읽어오는중..."):
        # 파일 확장자 분리
        name, ext = os.path.splitext(x)
        filePath = os.path.join(folder, x)
        # 코딩북인경우 readCodingBook실행
        if "codingbook" in name:
            questions_list += __readCodingBook(filePath)
    return Questions(questions_list)



from .datatype import Answer, Patient, Patients, Question, Questions
from .codingbook import read

__all__ = [
    "Answer",
    "Patient",
    "Patients",
    "Question",
    "Questions",
    #
    "read",
]
codingbook.__all__ = ["read"]



class Question:
    def __init__(
        self,
        survey_name: str = None,
        survey_name_korean: str = None,
        survey_code: str = None,
        has_options: str = None,  # o (option), e (no option)
        variable_type: str = None,  # n (number),v (string)
        variable_length: int = 0,
        question_text: str = None,
        question_type: str = None,  # s (single),m (multi)
        answer: dict = None,
    ):
        self.survey_name = survey_name
        self.survey_name_korean = survey_name_korean
        self.survey_code = survey_code.lower()
        self.has_options = has_options
        self.variable_type = variable_type
        self.variable_length = variable_length
        self.question_text = question_text
        self.question_type = question_type
        self.answer = answer
        pass

    @staticmethod
    def __parsePath(path):
        import os

        name = os.path.split(path)[-1]
        name, _ = os.path.splitext(name)
        file_type, data_type, year = name.split("_")
        return [file_type, data_type, year]

    def add_answer(self, row):
        answer = Answer.from_row(self, row)
        self.answer[answer.code] = answer.text

    def add_fileinfo(self, filePath):
        file_type, data_type, year = self.__parsePath(filePath)
        self.data_type = data_type
        self.year = year

    @classmethod
    def from_row(cls, row):
        import inspect

        dim = len(inspect.signature(cls.__init__).parameters)
        row = [(x or "").strip() for x in row[:dim]]
        question = cls(*row[: dim - 2])
        question.answer = {}
        if question.question_type == "m" or row[8].isnumeric():
            question.add_answer(row)
        return question

    def to_json(self):
        import json

        data = self.__dict__.copy()
        return json.dumps(data, indent=4, ensure_ascii=False)


class Answer:
    def __init__(
        self,
        question: Question,
        code: str,
        test: str,
    ):
        self.question = question
        self.code = code
        self.text = test

    @classmethod
    def from_row(cls, last_question, row):
        question = last_question
        return cls(question, row[8], row[9])


class Questions:
    def __init__(self, lst=None):
        self.list = lst if lst is not None else []
        self.len = len(self.list)
        self.valid_code = [
            "_".join(str.split(x.survey_code, "_")[1:])
            for x in self.list
            if hasattr(x, "survey_code")
        ]

    def keys(self, reverse=True, astype=list):
        import os

        def __sorting(a):
            if isinstance(a, str):
                a = str.split(a, " ")
            # data_type, year로 정렬
            return (a[0] == "track") * 100 + int(a[1])

        keys = []
        for d in set(self.__extract_attr("data_type")):
            for y in set(self.__extract_attr("year")):
                if not os.path.exists(f"./data_fixed/data_{d}_{y}.csv"):
                    continue
                if astype == list:
                    keys.append([d, y])
                else:
                    keys.append(f"{d} {y}")
        keys = list(sorted(keys, key=__sorting, reverse=reverse))
        return keys

    def append(self, x):
        self.list.append(x)

    @staticmethod
    def __filter(f, lst):
        return Questions(list(filter(f, lst)))

    @staticmethod
    def __find(f, lst):
        return next(filter(f, lst), None)

    def from_type(self, data_type, year):
        year = str(year).zfill(2)
        return self.__filter(
            lambda x: hasattr(x, "year")
            and hasattr(x, "data_type")
            and year == x.year
            and data_type == x.data_type,
            self.list,
        )

    def has_code(self, code):
        if isinstance(code, list) or isinstance(code, set):
            res = Questions()
            for x in code:
                f = lambda y: y.survey_code.endswith(f"_{x.lower()}")
                q = self.__find(f, self.list)
                res.append(q)
            return res
        return self.__filter(
            lambda x: x.survey_code.endswith(f"_{code.lower()}"),
            self.list,
        )

    def has_text(self, code):
        if isinstance(code, list):
            res = Questions()
            for x in code:
                q = self.__find(lambda y: x in y.question_text, self.list)
                res.append(q)
            return res
        return self.__filter(lambda x: code in x.question_text, self.list)

    @staticmethod
    def __extract_attr(self, name):
        return [getattr(x, name) if hasattr(x, name) else None for x in self.list]

    def __getattr__(self, name):
        if name not in self.__dict__:
            self.__dict__[name] = self.__extract_attr(self, name)
        return self.__dict__[name]

    def summary(self):
        res = "#### 실행결과  "
        year_list = sorted(set(self.year))

        res += (
            f"***\n"
            f"#### 1. 전체 질문데이터\n"
            f"***\n"
            f"- 전체 질문 데이터 **{len(self.list)}**개\n"
            f"- 코드 중복 제거시 **{len(set(self.valid_code))}**개\n"
            f"- 객관식 데이터 **{len([x for x in self.list if x.answer])}**개 / 주관식 데이터 **{len([x for x in self.list if not x.answer])}**개\n"
            f"- 연도별 질문 개수\n"
        )

        res += (
            f"||{'|'.join(year_list)}|\n"
            f"|:-:|{':-:|'*len(year_list)}\n"
            f"|baseline 질문 수|{'|'.join([str(self.from_type('baseline', year).len) for year in year_list])}|\n"
            f"|track 질문 수|{'|'.join([str(self.from_type('track', year).len) for year in year_list])}|\n"
        )

        res += (
            f"***\n"
            f"#### 3. 예시 데이터\n"
            f"***\n"
            f"- 질문 데이터\n"
            f"```json\n"
            f"{self.__find(lambda x:len(x.answer)>3, self.list).to_json()}\n"
            f"```\n"
        )
        return res


class Patient:
    def __init__(self, json):
        if json["socialno2"]:
            if json["socialno2"] != json["socialno2"] or "*" in json["socialno2"]:
                json["socialno2"] = None
        for k, v in json.items():
            setattr(self, k, v)

    def __eq__(self, other):
        if not isinstance(other, Patient):
            return False
        if self.cp and self.cp == other.cp:
            return True
        if self.socialno2 and self.socialno2 == other.socialno2:
            return True
        if self.name and self.name == other.name:
            if self.socialno1 and self.socialno1 == other.socialno1:
                return True
            if self.birthday and self.birthday == other.birthday:
                return True
        return False

    def __getattr__(self, name):
        if name not in self.__dict__:
            return None
        return self.__dict__[name]

    def to_json(self):
        return {
            "name": self.name,
            "birthday": self.birthday,
            "socialno1": self.socialno1,
            "socialno2": self.socialno2,
        }


class Patients:
    def __init__(self, list):
        self.list = list

    def append(self, p):
        self.list.append(p)

    def has_patient(self, patient):
        for p in self.list:
            if p == patient:
                return True
        return False



from setuptools import setup, find_packages

setup(
    name="pykoges",
    version="0.2.0",
    packages=find_packages(),
    install_requires=[],
    entry_points={},
    author="oculis",
    author_email="oculis0925@yonsei.ac.kr",
    description="Python module for cohort data reading",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/oculi-s/pykoges",
)
